/***************************************************************************
	mmavi - Classes for listing and creating AVI files

			-------------------
        begin           : 6/5/2004
        copyright       : (C) 2004 by Michael Minn
        email           : see www.michaelminn.com
                        -------------------

    This program is free software; you can redistribute it and/or modify
    it under the terms of version 2 of the GNU General Public License as 
    published by the Free Software Foundation.

***************************************************************************/

#ifndef MMAVI_H
#define MMAVI_H

#include <stdio.h>
#include "mmpnm.h"

class mmavi_container_chunk
{
	public:
	char chunk_name[4];
	int chunk_length;
	char content_type[4];
};

class mmavi_avi_header
{
	public:
	char chunk_name[4];
	int chunk_length;
	int microseconds_per_frame;
	int max_bytes_per_second;
	int reserved1;
	int flags;
	int total_frames;
	int initial_frames;
	int streams;
	int suggested_buffer_size;
	int width;
	int height;
	int scale;
	int rate;
	int start;
	int length;

	mmavi_avi_header() { memset(this, 0, sizeof(*this)); }
	int list()
	{
		printf("  AVI Header (%d/%d bytes)\n", chunk_length, sizeof(*this));
		printf("    ms/frame: %d\n", microseconds_per_frame);
		printf("    bytes/sec: %d\n", max_bytes_per_second);
		printf("    flags: %x\n", flags);
		printf("    total frames: %d\n", total_frames);
		printf("    initial frames: %d\n", initial_frames);
		printf("    streams: %d\n", streams);
		printf("    suggested buffer: %d\n", suggested_buffer_size);
		printf("    dimensions: %d x %d\n", width, height);
		if (scale)
			printf("    rate %d / scale %d = %d ticks/second\n", rate, scale, rate / scale);
		printf("    start: %d\n", start);
		printf("    length: %d\n", length);
		return 0;
	}
};

class mmavi_stream_header
{
	public:
	char chunk_name[4];
	int chunk_length;
	char stream_type[4];
	char stream_handler[4];
	int flags;
	int reserved;
	int initial_frames;
	int scale;
	int rate;
	int start;
	int length;
	int suggested_buffer_size;
	int quality;
	int sample_size;
	int reserved2[2];

	mmavi_stream_header() { memset(this, 0, sizeof(*this)); }
	int list()
	{
		printf("  Stream (%d/%d bytes)\n", chunk_length, sizeof(*this));
		printf("    Type: %.4s, handler %.4s\n", stream_type, stream_handler);
		printf("    flags: %d\n", flags);
		printf("    initial frames: %d\n", initial_frames);
		printf("    rate %d / scale %d = %d ticks/second\n", rate, scale, rate/ scale);
		printf("    start: %d\n", start);
		printf("    length: %d ticks\n", length);
		printf("    suggested buffer: %d\n", suggested_buffer_size);
		printf("    quality: %d\n", quality);
		printf("    sample size: %d\n", sample_size);
		return 0;
	}
};

class mmavi_image_format // bitmapinfoheader
{
	public:
	char chunk_name[4];
	int chunk_length;
	int structure_size;
	int width;
	int height;
	short planes; // always 1?
	short depth;
	int compression; // 0=rgb, 1=8-bit indexed, 2=4-bit indexed, 3=16/32bit rgb, 4=jpeg, 5=png
	int byte_size;
	int pixels_per_meter;
	int colors_used;
	int minimum_colors;
	char color_table[]; // array of b/g/r/a bytes

	mmavi_image_format() { memset(this, 0, sizeof(*this)); }
	int list()
	{
		printf("  Image format (%d/%d bytes)\n", chunk_length, chunk_length + 8);
		printf("    dimensions: %d x %d\n", width, height);
		printf("    structure size: %d\n", structure_size);
		printf("    planes: %d\n", planes);
		printf("    depth: %d\n", depth);
		printf("    compression algorithm: %d\n", compression);
		printf("    size: %d bytes\n", byte_size);
		printf("    resolution: %d/meter\n", pixels_per_meter);
		printf("    colors: %d (min %d)\n", colors_used, minimum_colors);
		return 0;
	}
};

class mmavi_sound_format // waveformatex
{
	public:
	char chunk_title[4];
        unsigned int chunk_length;
        unsigned short format_type;
        unsigned short channel_numbers;
        unsigned int sample_rate;
        unsigned int bytes_per_second;
        unsigned short bytes_per_sample;
        unsigned short bits_per_sample;

	mmavi_sound_format() { memset(this, 0, sizeof(*this)); }
	int list()
	{
		printf("  WAV Information (%d/%d bytes)\n", chunk_length, sizeof(*this));
		printf("    format: %d\n", format_type);
		printf("    channels: %d\n", channel_numbers);
		printf("    rate: %d\n", sample_rate);
		printf("    bytes/second: %d\n", bytes_per_second);
		printf("    bits/sample: %d\n", bits_per_sample);
		return 0;
	}
};

class mmavi_file
{
	public:
	mmavi_file() { memset(this, 0, sizeof(*this)); frames_per_second = 18; }
	int open_file(char *filename, int is_black_and_white, int frames_per_second);
	int add_frame(mmpnm_file *frame);
	int close_file();

	private:
	int descriptor;
	int frame_count;
	int frame_width;
	int frame_height;
	int is_black_and_white;
	int frames_per_second;

	int write_header();
};


int mmavi_list(char *filename);

#endif

