/***************************************************************************
	mmpnm - Utilities for loading PNM (Portable Anymap) files

			-------------------
        begin           : 6/4/2004
        copyright       : (C) 2004 by Michael Minn
        email           : see www.michaelminn.com
                        -------------------

    This program is free software; you can redistribute it and/or modify
    it under the terms of version 2 of the GNU General Public License as 
    published by the Free Software Foundation.

***************************************************************************/
#ifndef MMPNM_H
#define MMPNM_H

#include <stdio.h>
#include <string.h>
#include <math.h>

/************************************************************************
		Parameters for separating frames
************************************************************************/

#define FILM_WIDTH_1200_DPI         377
#define FRAME_HEIGHT_1200_DPI       200
#define FRAME_WIDTH_1200_DPI        276
#define SPROCKET_HEIGHT_1200_DPI     54
#define SPROCKET_WIDTH_1200_DPI      43
#define SPROCKET_AREA_WIDTH_1200_DPI 70
#define SOUND_AREA_WIDTH_1200_DPI    30
#define SPROCKET_TO_EDGE_1200_DPI    24

class mmpnm_framing
{
	public:
	int resolution_dpi;
	int film_width;
	int frame_width;
	int frame_offset;
	int sprocket_height;
	int sprocket_width;
	int sprocket_to_edge;
	int sprocket_area_width;
	int clip_top;
	int clip_bottom;
	int clip_left;
	int clip_right;
	int ignore_sprocket_holes;
	int frame_from_right;
	float frame_height;

	mmpnm_framing() { set_resolution(1200); }
	int set_resolution(int new_resolution);
};

class mmpnm_file
{
	public:
	int width;
	int height;
	unsigned char *color_data; // always 24-bit RGB
	unsigned char *gray_data;  // always 8-bit grayscale
	mmpnm_file *next; // for linked list of files/frames

	mmpnm_file() { memset(this, 0, sizeof(*this)); }
	~mmpnm_file();
	int add_flicker(int flicker_level);
	int add_grain(int grain_level, int grain_count);
	int add_jitter(float max_jitter);
	int add_noise(int max_noise);
	int add_pixel(int level, int x, int y, int color);
	int add_point(int level, float x, float y, int color);
	int add_to_list(mmpnm_file *&list);
	int adjust_color(int color, int low_value, int high_value);
	int clear();
	int convert_to_frames(mmpnm_file *&frames, mmpnm_framing &parameters);
	int copy(mmpnm_file *source);
	int crop(int new_width, int new_height);
	int delete_list();
	int load(char *filename);
	int copy_frame(mmpnm_file *source, int frame_width, int frame_height, int left_x, int top_y);
	int normalize();
	int save(char *file_name);
	int save_frames_to_separate_files(int is_black_and_white, int starting_frame_number);
	int shift(int x_shift, int y_shift);
	int shift(float x_shift, float y_shift);
};

#endif
