/***************************************************************************
	mmquicktime - Classes for creating Quicktime (.mov) files

			-------------------
        begin           : 6/4/2004
        copyright       : (C) 2004 by Michael Minn
        email           : see www.michaelminn.com
                        -------------------

    This program is free software; you can redistribute it and/or modify
    it under the terms of version 2 of the GNU General Public License as 
    published by the Free Software Foundation.

***************************************************************************/

#ifndef MMQUICKTIME_H
#define MMQUICKTIME_H

#include <stdio.h>
#include "mmpnm.h"

// Quicktime times are given in seconds since 1/1/1904.
// time() gives current time in seconds since 1/1/1970. 
// This is the difference.
#define MMQUICKTIME_TIME_OFFSET_SECONDS 2082758400

#define MMQUICKTIME_DEFAULT_FPS 18
#define MMQUICKTIME_TIME_SCALE  18000

#define MMQUICKTIME_COLOR_MODE_RGB  0
#define MMQUICKTIME_COLOR_MODE_GRAY 1

unsigned int little_endian(unsigned char *big_endian_value);
unsigned int little_endian_short(unsigned char *big_endian_value);
int big_endian(unsigned int little_endian, unsigned char *big_endian);
int big_endian_short(unsigned int little_endian, unsigned char *big_endian);
int mmquicktime_list(char *filename);
char *mmquicktime_date_string(unsigned int quicktime_time);

class mmquicktime_container
{
	public:
	unsigned char atom_size[4];
	unsigned char atom_name[4];
};

class mmquicktime_movie_header
{
	public:
	unsigned char atom_size[4];
	unsigned char atom_name[4]; // mvhd
	unsigned char version;
	unsigned char flags[3];
	unsigned char creation_time[4];     // Seconds since 1/1/1904
	unsigned char modification_time[4];
	unsigned char time_scale[4];        // Units per second
	unsigned char duration[4];
	unsigned char preferred_rate[4];
	unsigned char preferred_volume[2];
	unsigned char reserved[10];
	unsigned char matrix[36];
	unsigned char preview_time[4];
	unsigned char preview_duration[4];
	unsigned char poster_time[4];
	unsigned char selection_time[4];
	unsigned char selection_duration[4];
	unsigned char current_time[4];
	unsigned char next_track_id[4];
	void list()
	{
		printf("    Time scale: %d\n", little_endian(time_scale));
		printf("    Duration: %d (%d sec)\n", little_endian(duration),
			little_endian(duration) / little_endian(time_scale));
		printf("    Preferred rate: %f\n", little_endian(preferred_rate) / 65536.0); 
		printf("    Preferred volume: %d\n", little_endian_short(preferred_volume));
		printf("    Matrix: %f %f %f\n", 
			little_endian(&matrix[0]) / 65536.0,
			little_endian(&matrix[4]) / 65536.0,
			little_endian(&matrix[8]) / 65536.0);
		printf("            %f %f %f\n", 
			little_endian(&matrix[12]) / 65536.0,
			little_endian(&matrix[16]) / 65536.0,
			little_endian(&matrix[20]) / 65536.0);
		printf("            %f %f %f\n", 
			little_endian(&matrix[24]) / 65536.0,
			little_endian(&matrix[28]) / 65536.0,
			little_endian(&matrix[32]) / 65536.0);

		printf("    Preview time %d\n", little_endian(preview_time));
		printf("    Preview duration %d\n", little_endian(preview_duration));
		printf("    Poster time %d\n", little_endian(poster_time));
		printf("    Selection time %d\n", little_endian(selection_time));
		printf("    Selection duration %d\n", little_endian(selection_duration));
		printf("    Current time %d\n", little_endian(current_time));
		printf("    Next track ID %d\n", little_endian(next_track_id));
	}
};

class mmquicktime_track_header
{
	public:
	unsigned char atom_size[4];
	unsigned char atom_name[4]; // tkhd
	unsigned char version;
	unsigned char flags[3];
	unsigned char creation_time[4];
	unsigned char modification_time[4];
	unsigned char track_id[4];
	unsigned char reserved[4];
	unsigned char duration[4];
	unsigned char reserved2[8];
	unsigned char layer[2];
	unsigned char alternate_group[2];
	unsigned char volume[2];
	unsigned char reserved3[2];
	unsigned char matrix[36];
	unsigned char track_width[4];  // fixed-point: 16-bits whole, 16-bit fractional?
	unsigned char track_height[4]; // fixed-point
	void list()
	{
		printf("    Creation time %u (%s)\n", (unsigned int) little_endian(creation_time),
			mmquicktime_date_string((unsigned int) little_endian(creation_time)));
		printf("    Modification time %u (%s)\n", (unsigned int) little_endian(modification_time),
			mmquicktime_date_string((unsigned int) little_endian(modification_time)));
		printf("    Track %d\n", little_endian(track_id));
		printf("    Duration %d\n", little_endian(duration));
		printf("    Layer %d\n", little_endian_short(layer));
		printf("    Alternate Group %d\n", little_endian_short(alternate_group));
		printf("    Layer %02x%02x\n", layer[0], layer[1]);
		printf("    Alternate Group %02x%02x\n", alternate_group[0], alternate_group[1]);
		printf("    Volume %02x%02x\n", volume[0], volume[1]);
		printf("    %f x %f\n", little_endian(track_width) / 65536.0, 
			little_endian(track_height) / 65536.0);
	}
};

class mmquicktime_edit_list_entry
{
	public:
	unsigned char track_duration[4];
	unsigned char starting_time[4];
	unsigned char media_rate[4];
};

class mmquicktime_edit_list
{
	public:
	unsigned char atom_size[4];
	unsigned char atom_name[4]; // elst
	unsigned char version;
	unsigned char flags[3];
	unsigned char entry_count[4];
	mmquicktime_edit_list_entry entries[];
};

class mmquicktime_media_header
{
	public:
	unsigned char atom_size[4];
	unsigned char atom_name[4]; // mdhd
	unsigned char version;
	unsigned char flags[3];
	unsigned char creation_time[4];
	unsigned char modification_time[4];
	unsigned char time_scale[4];
	unsigned char duration[4];
	unsigned char language[2];
	unsigned char quality[2];
	void list()
	{
		printf("    Time scale: %d\n", little_endian(time_scale));
		printf("    Duration: %d (%d seconds)\n", little_endian(duration),
			little_endian(duration) / little_endian(time_scale));
		printf("    Language: %02x%02x\n", language[0], language[1]);
		printf("    Quality: %02x%02x\n", quality[0], quality[1]);
	}
};

class mmquicktime_media_handler_reference
{
	public:
	unsigned char atom_size[4];
	unsigned char atom_name[4]; // hdlr
	unsigned char version;
	unsigned char flags[3];
	unsigned char component_type[4];         // mhlr = media, dhlr = data
	unsigned char component_subtype[4];      // vide = video, soun = sound
	unsigned char component_manufacturer[4]; // usually appl (Apple)
	unsigned char component_flags[4];
	unsigned char component_flag_mask[4];
	char component_name[];
	void list()
	{
		printf("    Handler type %.4s, subtype %.4s, manufacturer %.4s\n",
			component_type, component_subtype, component_manufacturer);
		printf("    Flags %02x%02x%02x%02x\n", 
			component_flags[0], component_flags[1], component_flags[2], component_flags[3]);
		printf("    Flag Mask %02x%02x%02x%02x\n", 
			component_flag_mask[0], component_flag_mask[1], 
			component_flag_mask[2], component_flag_mask[3]);
		printf("    %.*s\n",
			little_endian(atom_size) - 28, component_name);
	}
};

class mmquicktime_video_media_information
{
	public:
	unsigned char atom_size[4];
	unsigned char atom_name[4]; // vmhd
	unsigned char version;
	unsigned char flags[3];
	unsigned char graphics_mode[2];
	unsigned char transfer_mode_red[2];
	unsigned char transfer_mode_green[2];
	unsigned char transfer_mode_blue[2];
	void list()
	{
		printf("    Graphics mode %02x%02x\n", graphics_mode[0], graphics_mode[1]);
		printf("    Transfer mode R=%02x%02x G=%02x%02x B=%02x%02x\n", 
			transfer_mode_red[0], transfer_mode_red[1],
			transfer_mode_green[0], transfer_mode_green[1],
			transfer_mode_blue[0], transfer_mode_blue[1]);
	}
};

class mmquicktime_sound_media_information
{
	public:
	unsigned char atom_size[4];
	unsigned char atom_name[4]; // smhd
	unsigned char version;
	unsigned char flags[3];
	unsigned char balance[2];
	unsigned char reserved[2];
};

class mmquicktime_data_reference_entry
{
	public:
	unsigned char atom_size[4];
	unsigned char atom_name[4]; // Data type: alis or rsrc
	unsigned char version;
	unsigned char flags[3];
	unsigned char data[];
};

class mmquicktime_data_reference
{
	public:
	unsigned char atom_size[4];
	unsigned char atom_name[4]; // dref
	unsigned char version;
	unsigned char flags[3];
	unsigned char entry_count[4];
	unsigned char data[]; // mmquicktime_data_reference_entry
	void list()
	{
		int size = little_endian(atom_size) - 16;
		printf("    %d data references (%d bytes)\n", little_endian(entry_count), size);
		/**
		unsigned char *scan = data; 
		while (scan < &data[size])
		{
			printf("      %.4s\n", &scan[4]);
			scan += little_endian(scan);
		}
		**/
	}
};

class mmquicktime_color_table 
{
	public:
	//unsigned char atom_size[4];
	//unsigned char atom_name[4]; // ctab
	unsigned char seed[4];
	unsigned char flags[2];
	unsigned char entry_count[2];
	unsigned char entries[2048]; // 256 entries, each with four 16-bit integers: ARGB
	void list()
	{
		printf("    %d color table entries\n", little_endian_short(entry_count) + 1);
		printf("    Seed %x\n", little_endian(seed));
	}
};


// From "Inside Macintosh: Quicktime" pp 3-448
class mmquicktime_sample_description_entry
{
	public:
	unsigned char atom_size[4];
	unsigned char atom_name[4];
	unsigned char reserved[6];
	unsigned char reference_index[2];
	unsigned char version[2];
	unsigned char revision_level[2];
	unsigned char vendor[4];
	unsigned char temporal_quality[4];
	unsigned char spacial_quality[4];
	unsigned char width[2];
	unsigned char height[2];
	unsigned char horizontal_resolution[4]; // fixed point
	unsigned char vertical_resolution[4];   // fixed point
	unsigned char data_byte_size[4];
	unsigned char frame_count[2];
	unsigned char compression_algorithm_name[32];
	unsigned char pixel_depth[2];
	unsigned char color_table_id[2];
	unsigned char color_table[]; // only present if color_table_id == 0
	
	void list()
	{
		printf("    Sample description: %.4s\n", atom_name);
		printf("    Index: %d\n", little_endian_short(reference_index));
		printf("    Version: %d\n", little_endian_short(version));
		printf("    Revision level: %d\n", little_endian_short(revision_level));
		printf("    Vendor: %.4s\n", vendor);
		printf("    Temporal quality: %x\n", little_endian(temporal_quality));
		printf("    Spacial quality: %x\n", little_endian(spacial_quality));
		printf("    Dimensions: %d x %d\n", little_endian_short(width), little_endian_short(height));
		printf("    Resolution: %f x %f\n", 
			little_endian(horizontal_resolution) / 65536.0,
			little_endian(vertical_resolution) / 65536.0);
		printf("    Data bytes: %d\n", little_endian(data_byte_size));
		printf("    Frame count: %d\n", little_endian_short(frame_count));
		printf("    Algorithm: %.32s\n", compression_algorithm_name);
		printf("    Pixel depth: %d\n", little_endian_short(pixel_depth));
		printf("    Color table ID: %d\n", little_endian_short(color_table_id));
		if (little_endian_short(color_table_id) == 0)
			((mmquicktime_color_table*)color_table)->list();
	}
};

class mmquicktime_sample_description
{
	public:
	unsigned char atom_size[4];
	unsigned char atom_name[4]; // stsd
	unsigned char version;
	unsigned char flags[3];
	unsigned char entry_count[4];
	unsigned char entries[];
	void list()
	{
		printf("    %d sample descriptions\n", little_endian(entry_count));
		for (int scan = 0, end = little_endian(atom_size) - 16; scan < end; 
			scan += little_endian(&entries[scan]))
			((mmquicktime_sample_description_entry*) &entries[scan])->list();
	}
};

class mmquicktime_time_to_sample_table
{
	public:
	unsigned char atom_size[4];
	unsigned char atom_name[4]; // stts
	unsigned char version;
	unsigned char flags[3];
	unsigned char entry_count[4];
	unsigned char entries[];
	void list()
	{
		printf("    %d time-to-sample entries\n", little_endian(entry_count));
		for (unsigned int entry = 0; entry < little_endian(entry_count); ++entry)
			printf("    %d samples, duration %d\n", little_endian(&entries[entry * 8]),
				little_endian(&entries[(entry * 8) + 4]));
	}
};

class mmquicktime_keyframe_table // AKA sync sample atom
{
	public:
	unsigned char atom_size[4];
	unsigned char atom_name[4]; // stss
	unsigned char version;
	unsigned char flags[3];
	unsigned char entry_count[4];
	unsigned char entries[];
	void list()
	{
		for (unsigned int entry = 0; entry < little_endian(entry_count); ++entry)
			printf("    Keyframe %d\n", little_endian(&entries[entry * 4]));
	}
};

class mmquicktime_chunk_table // AKA Sample-to-chunk
{
	public:
	unsigned char atom_size[4];
	unsigned char atom_name[4]; // stsc
	unsigned char version;
	unsigned char flags[3];
	unsigned char entry_count[4];
	unsigned char entries[];
	void list()
	{
		for (unsigned int entry = 0; entry < little_endian(entry_count); ++entry)
			printf("    First chunk %d, samples per chunk %d, description %x\n",
				little_endian(&entries[entry * 12]),
				little_endian(&entries[(entry * 12) + 4]),
				little_endian(&entries[(entry * 12) + 8]));
	}
};

class mmquicktime_sample_size_table 
{
	public:
	unsigned char atom_size[4];
	unsigned char atom_name[4]; // stsz
	unsigned char version;
	unsigned char flags[3];
	unsigned char sample_size[4]; // if 0, all samples have table entry; else this is size
	unsigned char entry_count[4];
	unsigned char entries[];
	void list()
	{
		printf("    Fixed sample size %d\n", little_endian(sample_size));
		printf("    %d sample size entries\n", little_endian(entry_count));
		for (unsigned int entry = 0; (entry < 10) && (entry < little_endian(entry_count)); ++entry)
			printf("    Sample %d: %d bytes\n", entry, little_endian(&entries[entry * 4]));
	}
};

class mmquicktime_sample_offset_table 
{
	public:
	unsigned char atom_size[4];
	unsigned char atom_name[4]; // stco
	unsigned char version;
	unsigned char flags[3];
	unsigned char entry_count[4];
	unsigned char entries[];
	void list()
	{
		printf("    %d chunk offset table entries\n", little_endian(entry_count));
		for (unsigned int entry = 0; (entry < 10) && (entry < little_endian(entry_count)); ++entry)
			printf("    Chunk %d @ %d\n", entry, little_endian(&entries[entry * 4]));
	}
};

class mmquicktime_file
{
	public:
	mmquicktime_file() { memset(this, 0, sizeof(*this)); descriptor = -1; }
	int open_file(char *filename, int color_mode, int frames_per_second);
	int add_frame(mmpnm_file *frame);
	int close_file();

	private:
	int color_mode;
	int descriptor;
	int frame_count;
	int frame_width;
	int frame_height;
	int frames_per_second;
};

#endif
